
<html>
<head>
<title>List Operations</title>
<LINK REL=STYLESHEET HREF="tcl_manual.css" TYPE="text/css">
</head>
<body bgcolor=white text=black>

<h2>List Operations</h2>

part of <a href="index.adp">Tcl for Web Nerds</a> 
by 
<a href="http://www-swiss.ai.mit.edu/~hal/hal.html">Hal Abelson</a>,
<a href="http://philip.greenspun.com/">Philip Greenspun</a>,
and
<a href="http://sandon.org/l/">Lydia Sandon</a>


<hr>


A Tcl list holds a sequence of elements, each of which can be a
number, a string, or another list.  Let's look at the commands for
constructing a list:

<blockquote>
<code><pre>
% # create an empty list using the list command  
% set user_preferences [list]
% # verify that we've created a 0-item list
% llength $user_preferences
0
% lappend user_preferences "hiking"
hiking
% lappend user_preferences "biking"
hiking biking
% lappend user_preferences "whale watching"
hiking biking {whale watching}
% llength $user_preferences
3
</pre></code>
</blockquote>


At this point, the variable <code>user_preferences</code> is a
three-element list.  We can pull individual items out with
<code>lindex</code>:

<blockquote>
<code><pre>
% lindex $user_preferences 0
hiking
% lindex $user_preferences 1
biking
% lindex $user_preferences 2
whale watching
% lindex $user_preferences 3
% lindex $user_preferences 5
</pre></code>
</blockquote>


Indexing is 0-based and <code>lindex</code> will return the empty
string rather than an error if you supply an out-of-range index.

<P>

When producing a page for a user, we'd be more likely to be interested
in searching the list.  The command <code>lsearch</code> returns the
index of the list element matching a query argument or -1 if
unsuccessful:

<blockquote>
<code><pre>
if { [lsearch -exact $user_preferences "hiking"] != -1 } {
    # look for new articles related to hiking 
}
</pre></code>
</blockquote>


Suppose that User A marries User B.  You want to combine their
preferences into a <code>household_preferences</code> variable using
the <code>concat</code> command:

<blockquote>
<code><pre>
% # use the multiple-argument form of list to create an N-element
% # list with one procedure call
% set spouse_preferences [list "programming" "computer games" "slashdot"]
programming {computer games} slashdot
% set household_preferences [concat $user_preferences $spouse_preferences]
hiking biking {whale watching} programming {computer games} slashdot
% llength $household_preferences
6
</pre></code>
</blockquote>


The Tcl shell's output is giving you an ugly insight into the internal
representation of lists as strings, with elements being separated by
spaces and grouped with braces.  There is no reason to rely on how Tcl
represents lists or even think about it.  Practice <em>data
abstraction</em> by using Tcl lists as your underlying storage
mechanism but define constructor and accessor procedures that the rest
of your source code invokes.  You won't find a huge amount of this
being done in Web development because the really important data
structures tend to be RDBMS tables, but here's an example of how it
might work, taken from <a
href="http://photo.net/philg/careers/four-random-people.tcl">http://photo.net/philg/careers/four-random-people.tcl</a>.
We're building a list of lists. Each sublist contains all the
information on a single historical figure.  Method A is quick and
dirty:


<blockquote>
<code><pre>
set einstein [list "A. Einstein" "Patent Office Clerk" "Formulated Theory of Relativity."]

set mill [list "John Stuart Mill" "English Youth" "Was able to read Greek and Latin at age 3."]

# let's build the big list of lists
set average_folks [list $einstein $mill ...]

# let's pull out Einstein's title 
set einsteins_title [lindex $einstein 1]
</pre></code>
</blockquote>


Method B uses data abstraction:

<blockquote>
<code><pre>
proc define_person {name title accomplishment} {
    return [list $name $title $accomplishment]
}

proc person_name {person} {
    return [lindex $person 0]
}

proc person_title {person} {
    return [lindex $person 1]
}

proc person_accomplishment {person} {
    return [lindex $person 2]
}

% set einstein [define_person "A. Einstein" "Patent Office Clerk" "Formulated Theory of Relativity."]
{A. Einstein} {Patent Office Clerk} {Formulated Theory of Relativity.}
% set einsteins_title [person_title $einstein]
Patent Office Clerk
</pre></code>
</blockquote>


Data abstraction will make building and maintaining a large system
much easier.  As noted above, however, the stateless nature of HTTP
means that any information you want kept around from page to page
must be kept by the RDBMS.  SQL already forces you to refer to data by
table name and column name rather than positionally.

<h3>Split and Join</h3>

Suppose we have a file
called <code>addressees.txt</code> with information about people, one
person to a line.  Suppose each of these lines contains, among other
information, an email address which we assume we can recognize by the
presence of an at-sign (<code>@</code>).  The following program
extracts all the email addresses and joins them together, separated by
commas and spaces, to form a string called <code>spam_address</code>
that we can use as the <code>Bcc:</code> field of an email message, to
spam them all:

<blockquote>
<code><pre>
# open the file for reading
set addressees_stream [open "~/addressees.txt" r]

# read entire file into a variable
set contents_of_file [read $addressees_stream] 

close $addressees_stream

# split the contents on newlines
set list_of_lines [split $contents_of_file "\n"]

# loop through the lines 
foreach line $list_of_lines {
    if { [regexp {([^ ]*@[^ ]*)} $line one_address] } {
        lappend all_addresses $one_address
    }
}

# use the join command to mush the list together
set bcc_line_for_mailer [join $all_addresses ", "]
</pre></code>
</blockquote>


Some things to observe here:

<ul>
<li>We've used the <code>foreach</code> operator (see the
chapter on <a href="control-structure.adp">control
structure</a>) to iterate over the list formed by splitting the file
at newline characters.

<li>We use pattern matching to extract the email address from each
line.  The pattern here specifies "stuff that doesn't contain a space,
followed by at-sign, followed by stuff that doesn't contain a space."
(See the explanation of
<code>regexp</code> in the chapter on <a
href="pattern-matching.adp">pattern matching</a>.)

<li>The iteration keeps <code>lappend</code>ing to
<code>all_addresses</code>, but this variable is never initialized.
<code>lappend</code> treats an unbound list variable the same as an empty list.
</ul>

<hr align=center width=50% size=5>

<h3>Reference: List operations</h3>

<UL>

<li><TT>list <I>arg1 arg2 ...</I></TT><BR>
<I>Construct and return a list the arguments.  Akin to
<CODE>(list arg1 arg2...)</CODE> in Scheme.</I>

<blockquote>
<code><pre>
set foo [list 1 2 [list 3 4 5]] <i>==> 1 2 {3 4 5}</i>
</pre></code>
</blockquote>


<a href="http://www.scriptics.com/man/tcl7.5/TclCmd/list.n.html">http://www.scriptics.com/man/tcl7.5/TclCmd/list.n.html</a>

<br>
<br>

<LI><CODE>lindex <I>list i</I></CODE><BR>
<I>Returns the ith element from list; starts at index 0.
<blockquote>
<code><pre>
lindex $foo 0 <i>==> 1</i>
</pre></code>
</blockquote>

</I>

<a href="http://www.scriptics.com/man/tcl7.5/TclCmd/lindex.n.html">http://www.scriptics.com/man/tcl7.5/TclCmd/lindex.n.html</a>

<br>
<br>


<LI><CODE>llength <I>list</I></CODE><BR>
<I>Returns the number of elements in list.</I>
<blockquote>
<code><pre>
llength $foo <i>==> 3</i>
</pre></code>
</blockquote>


<a href="http://www.scriptics.com/man/tcl7.5/TclCmd/llength.n.html">http://www.scriptics.com/man/tcl7.5/TclCmd/llength.n.html</a>

<br>
<br>

<LI><CODE>lrange <I>list i j</I></CODE><BR>
<I>Returns the ith through jth elements from list.</I>
<blockquote>
<code><pre>
lrange $foo 1 2 <i>==> 2 {3 4 5}</i>
</pre></code>
</blockquote>


<a href="http://www.scriptics.com/man/tcl7.5/TclCmd/lrange.n.html">http://www.scriptics.com/man/tcl7.5/TclCmd/lrange.n.html</a>

<br>
<br>

<LI><CODE>lappend <I>listVar arg arg...</I> </CODE><BR>
<I>Append elements to the value of listVar and reset listVar to the new
list. Please note that 
<i>listVar</i> is the name of a variable and not the value, i.e., 
you should not put a $ in front of it (the same way that <code>set</code> works.</I>
<blockquote>
<code><pre>
lappend foo [list 6 7] <i>==> 1 2 {3 4 5} {6 7}</i>
set foo <i>==> 1 2 {3 4 5} {6 7}</i>
</pre></code>
</blockquote>


<a href="http://www.scriptics.com/man/tcl7.5/TclCmd/lappend.n.html">http://www.scriptics.com/man/tcl7.5/TclCmd/lappend.n.html</a>

<br>
<br>

<LI><CODE>linsert <I>list index arg arg...</I> </CODE><BR>
<I>Insert elements into list before the element at position index.
Returns a new list.</I>
<blockquote>
<code><pre>
linsert $foo 0 0 <i> ==> 0 1 2 {3 4 5} {6 7}</i>
</pre></code>
</blockquote>


<a href="http://www.scriptics.com/man/tcl7.5/TclCmd/linsert.n.html">http://www.scriptics.com/man/tcl7.5/TclCmd/linsert.n.html</a>

<br>
<br>

<LI><CODE>lreplace <I>list i j arg arg...</I> </CODE><BR>
<I>Replace elements i through j of list with the args.  Returns a new
list and leaves the original list unmodified.</I>
<blockquote>
<code><pre>
lreplace $foo 3 4 3 4 5 6 7 <i>==> 0 1 2 3 4 5 6 7</i>
set foo <i>==> 1 2 {3 4 5} {6 7}</i>
</pre></code>
</blockquote>


<a href="http://www.scriptics.com/man/tcl7.5/TclCmd/lreplace.n.html">http://www.scriptics.com/man/tcl7.5/TclCmd/lreplace.n.html</a>

<br>
<br>

<LI><CODE>lsearch <I>mode list value</I></CODE><BR>
<I>Return the index of the element in list that matches the value
according to the mode, which is -exact, -glob, or -regexp. -glob is
the default.  Return -1 if not found.</I>
<blockquote>
<code><pre>
set community_colleges [list "caltech" "cmu" "rpi"]
lsearch -exact $community_colleges "caltech" <i>==> 0</i>
lsearch -exact $community_colleges "harvard" <i>==> -1</i>
</pre></code>
</blockquote>


<a href="http://www.scriptics.com/man/tcl7.5/TclCmd/lsearch.n.html">http://www.scriptics.com/man/tcl7.5/TclCmd/lsearch.n.html</a>

<br>
<br>

<LI><CODE>lsort <I>switches list</I></CODE><BR>
<I>Sort elements of the list according to the switches: -ascii,
-integer, -real, -increasing, -decreasing, -command <I>command</I>.
Returns a new list.</I>
<blockquote>
<code><pre>
set my_friends [list "herschel" "schlomo" "mendel"]
set my_sorted_friends [lsort -decreasing $my_friends] <i>==> schlomo mendel herschel </i>
</pre></code>
</blockquote>


<a href="http://www.scriptics.com/man/tcl7.5/TclCmd/lsort.n.html">http://www.scriptics.com/man/tcl7.5/TclCmd/lsort.n.html</a>

<br>
<br>

<LI><CODE>concat <I>arg arg...</I></CODE><BR>
<I>Join multiple lists together into one list.</I>
<blockquote>
<code><pre>
set my_wifes_friends [list "biff" "christine" "clarissa"]
concat $my_wifes_friends $my_friends <i>==> biff christine clarissa herschel schlomo mendel</i>
</pre></code>
</blockquote>


<a href="http://www.scriptics.com/man/tcl7.5/TclCmd/concat.n.html">http://www.scriptics.com/man/tcl7.5/TclCmd/concat.n.html</a>

<br>
<br>

<LI><CODE>join <I>list joinString</I></CODE><BR>
<I>Merge the elements of list to produce a string, where the original
list elements are separated by 
joinString.  Returns the resulting string.</i> 
<blockquote>
<code><pre>
set foo_string [join $foo ":"] <i>==> 0:1:2:3:4:5:6:7</i>
</pre></code>
</blockquote>


<a href="http://www.scriptics.com/man/tcl7.5/TclCmd/join.n.html">http://www.scriptics.com/man/tcl7.5/TclCmd/join.n.html</a>

<br>
<br>

<LI><CODE>split <I>string splitChars</I></CODE><BR>
<I>Split a string to produce a list, using (and discarding) the
characters in splitChars as the places where to split. Returns the
resulting list.</I>
<blockquote>
<code><pre>
set my_ip_address 18.1.2.3 <i>==> 18.1.2.3</i>
set ip_elements [split $my_ip_address "."] <i>==> four element list,
with values 18 1 2 3</i>
</pre></code>
</blockquote>


<a href="http://www.scriptics.com/man/tcl7.5/TclCmd/split.n.html">http://www.scriptics.com/man/tcl7.5/TclCmd/split.n.html</a>

<br>
<br>

</UL>



<br>
<br>

Continue on to <A HREF="pattern-matching.adp">Pattern matching</A>.



<hr>
Return to <a href="index.adp">Table of Contents</a>
<br>
<br>
<address><a href="mailto:lsandon@alum.mit.edu">lsandon@alum.mit.edu</a></address>

<center><h3>Reader's Comments</h3></center>
	<blockquote>
		There is a way of sorting strings according to lists inside the list. If you have a list of items and values, you can then sort by the values. <P>
See: <a href="http://tcl.activestate.com/man/tcl8.2.3/TclCmd/ lsort.htm#M12">http://tcl.activestate.com/man/tcl8.2.3/TclCmd/ lsort.htm#M12</a> and <a href="http://openacs.org/bboard/q-and-a-fetch-msg.tcl?msg_id=0005vw&topic_id=11&topic=OpenACS">http://openacs.org/bboard/q-and-a-fetch-msg.tcl?msg_id=0005vw&topic_id=11&topic=OpenACS</a>

		<br><br>

-- <A HREF="/shared/community-member?user_id=167999">Jade Rubick</a>, August 23, 2002</blockquote>

	<center><a href="/comments/add?page_id=4731">Add a comment</a> | <a href="/links/add?page_id=4731">Add a link</a></center>

</body>
</html>


